function prepareFormsToValidate(formsToValidate) {
    formsToValidate.forEach(function (formInfo) {
        let formId = formInfo.formId;
        $(`#${formId}>div>input`).on('input', function () {
            let inputId = $(this).get(0).getAttribute('id');
            validateForm(formInfo, inputId);
        });
    });
}

function getFormInfo(formId) {
    let selectedForm = null;
    formsToValidate.forEach(function (formInfo) {
        if (formInfo.formId === formId)
            selectedForm = formInfo;
    });
    return selectedForm;
}

const ERROR_UNDER_MINIMUM_LENGTH = "UNDER_MINIMUM_LENGTH";
const ERROR_OVER_MAXIMUM_LENGTH = "OVER_MAXIMUM_LENGTH";
const ERROR_CONTAIN_SPACES = "CONTAIN_SPACES";

const EMAIL_REGEX = {
    regex: [
        [0, "VALID_EMAIL", "^[\\w-\\.]+@([\\w-]+\\.)+[\\w-]{2,4}$"],
        [-1, "INVALID_EMAIL"]
    ]
};

const BLOCKSPACE_REGEX = "^\\S*$";

function validateForm(formInfo, inputId) {
    let specificInput = inputId != null;

    let form = document.getElementById(formInfo.formId);
    let inputs = formInfo.inputs;

    let formValid = true;

    inputs.forEach(function (inputInfo) {

        let valid = true;

        let errors = [];
        let regexStatus = [0, "NONE_REGEX"];

        if (specificInput && inputInfo.inputId != inputId)
            return;

        let input = form[inputInfo.inputId];
        let alertElement = document.getElementById(inputInfo.alertId);

        if ((inputInfo.inputType == undefined || inputInfo.inputType == null) && input.getAttribute('type') != null)
            inputInfo.inputType = input.getAttribute('type');

        if (alertElement != null)
            alertElement.innerText = null;

        let range = [-1, -1];

        if (inputInfo.minLength != undefined && inputInfo.minLength != null)
            range[0] = inputInfo.minLength;
        if (inputInfo.maxLength != undefined && inputInfo.maxLength != null)
            range[1] = inputInfo.maxLength;

        let belowMinimumLength = (range[0] != -1 && input.value.length < range[0]);
        let overMaximumLength = (range[1] != -1 && input.value.length > range[1]);

        if (belowMinimumLength || overMaximumLength) {
            if (belowMinimumLength) { errors.push(ERROR_UNDER_MINIMUM_LENGTH) };
            if (overMaximumLength) { errors.push(ERROR_OVER_MAXIMUM_LENGTH) };
            if (alertElement != null)
                if (inputInfo.lengthMessage == undefined)
                    alertElement.innerText = `It must to be between ${range[0]} and ${range[1]} characters!`;
                else
                    alertElement.innerText = inputInfo.lengthMessage.replace('%min', range[0]).replace('%max', range[1]);
        }

        if ((inputInfo.regex == undefined || inputInfo.regex == null) && inputInfo.inputType.toLowerCase() == "email")
            inputInfo.regex = EMAIL_REGEX.regex;

        if (inputInfo.blockSpace != undefined && inputInfo.blockSpace != null && inputInfo.blockSpace) {
            let spaceRegex = new RegExp(BLOCKSPACE_REGEX);
            if (!spaceRegex.test(input.value)) {
                errors.push(ERROR_CONTAIN_SPACES);
                if (alertElement != null)
                    if (inputInfo.blockSpaceMessage == undefined)
                        alertElement.innerText = `Space is not allowed.`;
                    else
                        alertElement.innerText = inputInfo.blockSpaceMessage;
            }
        }

        if (inputInfo.regex != undefined && inputInfo.regex != null) {
            let regexPriority = [-1, "UNDEFINED_REGEX_FAIL"];
            let customFailReason = null;
            inputInfo.regex.forEach(function (regexInfo) {
                let priority = regexInfo[0];
                let description = regexInfo[1];
                if (priority == -1) {
                    customFailReason = description;
                    return;
                }
                let regex = new RegExp(regexInfo[2].replace('%min', inputInfo.minLength).replace('%max', inputInfo.maxLength));
                if (regex.test(input.value) && priority > regexPriority[0])
                    regexPriority = [priority, description];
            });
            if (regexPriority[0] == -1) {
                if (customFailReason != null)
                    regexPriority[1] = customFailReason;
                if (alertElement != null)
                    if (inputInfo.regexMessage == undefined)
                        alertElement.innerText = `Please enter a valid format!`;
                    else
                        alertElement.innerText = inputInfo.regexMessage;
            }
            regexStatus = regexPriority;
        }

        if (inputInfo.required != undefined && inputInfo.required && input.value.length == 0) {
            errors.push("REQUIRED");
            if (alertElement != null)
                if (inputInfo.requiredMessage == undefined)
                    alertElement.innerText = "Required!";
                else
                    alertElement.innerText = inputInfo.requiredMessage;
        }

        if (inputInfo.customFilter != undefined && inputInfo.customFilter != null) {
            let additionalErrors = inputInfo.customFilter(input.value);
            additionalErrors.forEach(function (additionalError) {
                errors.push(additionalError);
            });
        }

        if (errors.length > 0 || regexStatus[0] < 0)
            valid = false;

        if (!valid)
            formValid = false;

        if (valid && alertElement != null)
            if (inputInfo.validatedMessage == undefined)
                alertElement.innerText = null;
            else
                alertElement.innerText = inputInfo.validatedMessage;

        if (inputInfo.listener != undefined && inputInfo.listener != null)
            inputInfo.listener(valid, errors, regexStatus);
    });

    return formValid;
}